package ch.hepia.api.transport;

import org.json.JSONObject;

import java.sql.Time;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;

public class Stop {
    private Location station;
    private Time arrival;
    private Time departure;
    private int delay;
    private int plateform;
    private Prognosis prognosis;

    /**
     * Default Stop in Geneva
     */
    private final static class DefaultStop extends Stop{
        /**
         * Constructor of DefaultStop
         * DefaultStop is located in Geneva
         * No Time of departure nor Arrival, no Prognosis, no plateform and no delay
         * @throws ParseException
         */
        private DefaultStop() throws ParseException {
            super(new Location.LocationBuilder(new JSONObject(), "refine").build(),
                    new Time(new SimpleDateFormat("dd:kk:mm:ss").parse("00:00:00:00").getTime()),
                    new Time(new SimpleDateFormat("dd:kk:mm:ss").parse("00:00:00:00").getTime()),
                    0, 0,
                    new Prognosis.PrognosisBuilder(new JSONObject()).build());
        }
    }

    /**
     * Constructor of the Stop class
     * @param station Location where the Stop is
     * @param arrival time of arrival at this Stop
     * @param departure time of departure from this Stop
     * @param delay duration time of unpredictated delay
     * @param plateform where the Stop will arrive to and depart from
     * @param prognosis complementary informations (See also {@link Prognosis})
     */
    private Stop(Location station, Time arrival, Time departure, int delay, int plateform, Prognosis prognosis){
        this.station = station;
        this.arrival = arrival;
        this.departure = departure;
        this.delay = delay;
        this.plateform = plateform;
        this.prognosis = prognosis;
    }

    /**
     * Get the Location of the Stop
     * @return station
     */
    public Location getLocation(){
        return station;
    }

    /**
     * Get the departure time of the transport from the Stop
     * @return departure
     */
    public Time getDepartureTime(){
        return departure;
    }

    /**
     * Get the Arrival time of the transport to the Stop
     * @return arrival
     */
    public Time getArrivalTime(){
        return arrival;
    }

    /**
     * Builder of Stop objects
     */
    public final static class StopBuilder{
        private JSONObject datas;

        /**
         * Constructor of the class StopBuilder
         * Stock Datas from a JSON object given
         * @param datas contains datas to construct a Stop object
         */
        public StopBuilder(JSONObject datas){
            this.datas = datas;
        }

        /**
         * Build a Stop object from the datas obtained
         * @return a new Stop object
         * @throws ParseException
         */
        public Stop build() throws ParseException {
            if(datas.isEmpty()){
                return new DefaultStop();
            }

            //getting the first key to know the type of the Location
            String[] keys = (String[]) datas.keySet().toArray();
            String type = keys[0];
            JSONObject locationJSON = datas.getJSONObject(type);
            Location.LocationBuilder location = new Location.LocationBuilder(locationJSON, type);

            DateFormat formatter = new SimpleDateFormat("yyyy-MM-dddkk:mm:ss");

            String arrivalString = datas.isNull("arrival") ? "0000-00-00000:00:00" : datas.getString("arrival");
            Time arrival = new Time(formatter.parse(arrivalString).getTime());
            String depatureString = datas.isNull("departure") ? "0000-00-00000:00:00" : datas.getString("departure");
            Time departure = new Time(formatter.parse(depatureString).getTime());

            int delay = datas.isNull("delay") ? 0 : datas.getInt("delay");
            int plateform = datas.isNull("plateform") ? 0 : datas.getInt("plateform");

            JSONObject prognosisJSON = new JSONObject(datas.get("prognosis"));
            Prognosis.PrognosisBuilder prognosis = new Prognosis.PrognosisBuilder(prognosisJSON);

            return new Stop(location.build(), arrival, departure, delay, plateform, prognosis.build());
        }
    }
}
